/**
 * DataWorkflowTest.java
 */
package com.sample;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.naming.InitialContext;
import javax.persistence.EntityManagerFactory;
import javax.transaction.UserTransaction;

import org.drools.process.instance.WorkItemHandler;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.process.ProcessInstance;
import org.drools.runtime.process.WorkItem;
import org.drools.runtime.process.WorkItemManager;
import org.jbpm.process.audit.JPAWorkingMemoryDbLogger;

import bitronix.tm.resource.jdbc.PoolingDataSource;

/**
 * Test cases for the Data Workflow.
 */
public class DataWorkflowTestV2 extends BaseProcessTest {

	/**
	 * Run the tests for the Data Workflow.
	 */
	public static void main(String[] args) {
		DataWorkflowTestV2 dataWorkflowTest = new DataWorkflowTestV2();
		
		try {
			
			dataWorkflowTest.runTests();
			
		} catch (Exception e) {
			
			//TODO some proper logging
			e.printStackTrace();
			
		}
	}
	
	public void runTest() throws Exception {
		
		//Set up the persistence properties for the test session
		setDestConnectionString("jdbc:oracle:thin:@10.2.0.105:1521:BCGBDEV8");
		setDestUserName("AUTOSALVAGE_JBPM");
		setDestPassword("passw0rd");
		setDataSourceUniqueName("jdbc/processInstanceDS");

		PoolingDataSource dateSource = getJTAPoolingDatasource();
		
		EntityManagerFactory emf = getEntityManagerFactory("org.jbpm.persistence.jpa");
		
		StatefulKnowledgeSession session = getKnowledgeSession(emf, "data-v0.0.2.bpmn2");
		JPAWorkingMemoryDbLogger logger = attachLogger(session);

		//Attach any customised workflow action listeners for this workflow
		session.getWorkItemManager().registerWorkItemHandler("Log", new WorkItemHandler() {
			
			@Override
			public void executeWorkItem(WorkItem workItem, WorkItemManager manager) {
				System.out.println("This might be some kind of domain specific handler being executed.");
				manager.completeWorkItem(workItem.getId(), null);
			}
			
			@Override
			public void abortWorkItem(WorkItem arg0, WorkItemManager arg1) {
				System.out.println("This might be some kind of domain specific handler being aborted.");
				
			}
		});
		
		//Start a JTA transaction for the workflow activity
		InitialContext ictx = new InitialContext();
		
		UserTransaction ut = (UserTransaction) ictx.lookup("java:comp/UserTransaction");

		ut.begin();

		//Do the test cases
		
		Map<String, Object> processVariables = new HashMap<String, Object>();
		
		processVariables.put("tier", "Two");
		
		ProcessInstance process = session.startProcess("com.bluecycle.zephyr.workflow.data-v0.0.2", processVariables);

		session.signalEvent("Signal1", null, process.getId());	

		session.signalEvent("Signal2", null, process.getId());	

		ut.commit();
		
		logger.dispose();
		session.dispose();
		dateSource.close();
		
	}
	
}
